require_relative 'configurable/version'

module RubyRising
  module Configurable
    class UnconfigurableTypeError < StandardError; end

    def self.extended(base)
      if base.class == Module
        add_configuration_methods_to(base)
      else
        raise UnconfigurableTypeError
      end
    end #NT

    def self.add_configuration_methods_to(base)
      base.instance_eval do
        def configuration
          @configuration ||= self::Configuration.new
        end #TT
        def reset!
          @configuration = self::Configuration.new
        end #TT
        def configure
          yield(configuration)
        end #TT
      end
    end
  end
end
