class Enum
  attr_reader :index, :type
  def initialize(type, &block)
    @type = type
    @index = 0
    self.instance_eval(&block)
  end

  def instance_variable_set(*args)
    raise SimpleSkills::Enum::AttemptedToAlterLockedEnum, "Cannot set instance variables on a locked enum" if @locked
    super(*args)
  end

  def expand(&block)
    self.instance_eval(&block)
  end

  def lock!
    @locked = true
  end

  def options
    @options ||= {}
  end

  def override(option, with:)
    raise SimpleSkills::Enum::AttemptedToAlterLockedEnum, "Cannot override options of a locked enum" if @locked
    options[option].set with
  end

  def option(sym)
    raise SimpleSkills::Enum::AttemptedToAlterLockedEnum, "Cannot add options to a locked enum" if @locked
    raise SimpleSkills::Enum::AttemptedToChangeDefinedOption, "The option #{sym} already exists on enum.\nUse Enum#override(:#{sym}, with: [new value here]) if you want to override #{sym}" if options[sym]
    new_option = EnumOption.new(sym, self)
    new_option.raw_value = sym.to_s if type == String
    new_option.raw_value = index and @index += 1 if type == Integer
    options[sym] = new_option
  end

  def [](option)
    options[option] or raise SimpleSkills::Enum::MissingOption, "No option #{option} is defined on enum."
  end
end
